#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <optcfg.h>
#include "opt.h"

Options opt;

Options def_opt = {
	1280, 800,
	false,	// vr
	false,	// fullscreen
	0,		// scene file
	true,	// music
	true	// reflections
};

enum {
	OPT_SIZE,
	OPT_VR,
	OPT_FULLSCREEN,
	OPT_WINDOWED,
	OPT_SCENEFILE,
	OPT_MUSIC,
	OPT_REFLECT,
	OPT_HELP
};

static optcfg_option options[] = {
	// short, long, id, desc
	{'s', "size", OPT_SIZE, "window size (WxH)"},
	{0, "vr", OPT_VR, "enable VR mode"},
	{'f', "fullscreen", OPT_FULLSCREEN, "run in fullscreen mode"},
	{'w', "windowed", OPT_WINDOWED, "run in windowed mode"},
	{0, "scene", OPT_SCENEFILE, "scene file to open"},
	{'m', "music", OPT_MUSIC, "play background audio"},
	{'r', "reflect", OPT_REFLECT, "render reflections"},
	{'h', "help", OPT_HELP, "print usage and exit"},
	OPTCFG_OPTIONS_END
};

static int opt_handler(optcfg *oc, int opt, void *cls);
static int arg_handler(optcfg *oc, const char *arg, void *cls);

bool init_options(int argc, char **argv, const char *cfgfile)
{
	// default options
	opt = def_opt;

	optcfg *oc = optcfg_init(options);
	optcfg_set_opt_callback(oc, opt_handler, 0);
	optcfg_set_arg_callback(oc, arg_handler, 0);

	if(cfgfile) {
		optcfg_parse_config_file(oc, cfgfile);
	}

	if(argv && optcfg_parse_args(oc, argc, argv) == -1) {
		fprintf(stderr, "invalid option\n");
		optcfg_destroy(oc);
		return false;
	}

	optcfg_destroy(oc);
	return true;
}

static bool is_enabled(optcfg *oc)
{
	int res;
	optcfg_enabled_value(oc, &res);
	return res != 0;
}

static int opt_handler(optcfg *oc, int optid, void *cls)
{
	switch(optid) {
	case OPT_SIZE:
		{
			char *valstr = optcfg_next_value(oc);
			if(!valstr || sscanf(valstr, "%dx%d", &opt.width, &opt.height) != 2) {
				fprintf(stderr, "size must be in the form: WIDTHxHEIGHT\n");
				return -1;
			}
		}
		break;

	case OPT_VR:
		opt.vr = is_enabled(oc);
		break;

	case OPT_FULLSCREEN:
		opt.fullscreen = is_enabled(oc);
		break;

	case OPT_WINDOWED:
		opt.fullscreen = !is_enabled(oc);
		break;

	case OPT_SCENEFILE:
		opt.scenefile = strdup(optcfg_next_value(oc));
		break;

	case OPT_MUSIC:
		opt.music = is_enabled(oc);
		break;

	case OPT_REFLECT:
		opt.reflect = is_enabled(oc);
		break;

	case OPT_HELP:
		printf("Usage: demo [options]\nOptions:\n");
		optcfg_print_options(oc);
		exit(0);
	}
	return 0;
}

static int arg_handler(optcfg *oc, const char *arg, void *cls)
{
	if(opt.scenefile) {
		fprintf(stderr, "unexpected argument: %s\n", arg);
		return -1;
	}
	opt.scenefile = arg;
	return 0;
}
